#' Usage:
#' Initial run
#' Rscript ./R/edl_feeding_crawler.R
#' To update data
#' Rscript ./R/edl_feeding_crawler.R update

library(httr)
library(xml2)
library(dplyr)
library(stringi)
library(reshape)
library(reshape2)
library(parallel)
library(pbapply)
pboptions(type = "txt")
ncores <- ifelse(detectCores() >= 8, 8, detecCores())

args <- commandArgs(trailingOnly = TRUE)

if(length(args) == 0){
  update = FALSE
}else{
  update = ifelse(args[1] == "update", TRUE, FALSE)
}

if(update){
  load("./data/edl_feeding_crawler_output.RData", verbose = T)
  feeding_all_old <- feeding_all
  outlet_meta_old <- outlet_meta
  stations_dt_old <- stations_dt
  date1 <- max(lubridate::as_date(feeding_all_old$dateoffeeding) + 1)
}else{
  date1 <- "2008-10-20"
}

date2 <- as.character(lubridate::today())

# get station names
stations <- read_xml("./data/substations.xml") # manually derived by Maroun Bou Sleiman
stations_dt <- data.frame(station_name = xml2::xml_text(xml2::xml_children(stations)),
                          station_id = unlist(xml2::xml_attrs(xml2::xml_children(stations))), stringsAsFactors = F)
httr_handle <- handle("http://www.edl.gov.lb/")
# get outlets and associated metadata of stations
outlet_meta <- pblapply(X=stations_dt$station_id, FUN = function(x){
  print(x)
  bd <- list(mode = "feeders-sel2", id = x)
  res <- POST(handle = httr_handle, path = "feedingdata.php", body = bd)
  # http_type(res)
  substations <- content(res, as = "parsed")
  substations_metadata <- jsonlite::fromJSON(xml_text(xml2::xml_child(substations,1), trim = T))$results
  Encoding(substations_metadata$text) <- "UTF-8"
  substations_metadata$station_id <- x
  substations_metadata
}, cl = ncores )

outlet_meta <- do.call(rbind, outlet_meta)
dir.create("./data/feeding_all_tmp")



getRes <- function(querybody){
  res2 <-  POST(handle = httr_handle, path = "feedingdata.php", body = querybody)
  results <- content(res2, as = "parsed")
  results <- jsonlite::fromJSON(xml_text(xml2::xml_child(results,1), trim = T))$list_feeders
  return(results)
}

feeding_all <-pblapply(X=unique(outlet_meta$id), FUN = function(x){
  print(x)
  bd2 <- list(mode = "load", actpre = "act", a_feeders = x, d1 = date1, d2=date2)
  results <- getRes(querybody = bd2)
  while(length(results) == 0){
    results <- getRes()
  }
  write.table(results, file = paste0("./data/feeding_all_tmp/", x,".tmp"))
  results
}, cl = ncores)
feeding_all <- do.call(rbind, feeding_all)


if(update){
  feeding_all <- rbind(feeding_all_old, feeding_all)
}


save(feeding_all, outlet_meta, stations_dt, file = "./data/edl_feeding_crawler_output.RData")
#alternatively, save individual files in csv
write.csv(feeding_all, file = "./data/feeding_all.csv")
write.csv(outlet_meta, file = "./data/outlet_meta.csv")
write.csv(stations_dt, file = "./data/stations_dt.csv")



# library(ggplot2)
# library(lubridate)
# timecols <- paste0("time", 0:23)
# feeding_all_reshaped <- reshape(data = feeding_all, idvar = "id", varying = list(which(colnames(feeding_all) %in% timecols)), direction = "long",  v.names = "power")
# feeding_all_reshaped$dateoffeeding_lub <- as_datetime(ymd(feeding_all_reshaped$dateoffeeding))
# hour(feeding_all_reshaped$dateoffeeding_lub) <- feeding_all_reshaped$time
# table(feeding_all_reshaped$power)
# table(feeding_all_reshaped$subgroup)
# table(feeding_all_reshaped$substationid)


# p <- ggplot(feeding_all_reshaped, aes(x = dateoffeeding_lub, y = feedername, fill = power)) +
#   geom_tile() + scale_fill_manual(values = c("0" = "red", "1" = "green")) +
#   facet_wrap(substationname~., drop= T, ncol = 2)
# 
# ##  theme(strip.text.y.left = element_text(angle = 0))
# ggsave(p, file = "./feeding_heatmap.png", width = 20, height = 30)
# 
# 
# feeding_all_reshaped_daily <- plyr::ddply(feeding_all_reshaped, c("substationname", "substationid","feedername", "fid", "dateoffeeding"), plyr::summarize, hours_on = sum(power == "1"))
# feeding_all_reshaped_daily$dateoffeeding_lub <- ymd(feeding_all_reshaped_daily$dateoffeeding)
# p2 <- ggplot(feeding_all_reshaped_daily, aes(x = dateoffeeding_lub, y = hours_on, group = feedername, color = hours_on)) +
#   geom_line() +
#   # scale_color_continuous(low = "red", high = "green") +
#   facet_wrap(substationname~., drop= T, ncol = 4)
